# API Docs - v2.0.0

## Approximate

### count *<a target="_blank" href="http://siddhi.io/documentation/siddhi-5.x/query-guide-5.x/#stream-processor">(Stream Processor)</a>*

<p style="word-wrap: break-word">This extension applies the <code>count-min sketch</code> algorithm to a Siddhi window. The algorithm calculates the approximate count i.e., the frequency of events that arrive, based on  the given values for the 'relative error' and 'confidence value'. Note that, using this extension without a window may cause an 'out of memory' error.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
approximate:count(<INT|DOUBLE|FLOAT|LONG|STRING|BOOL|TIME|OBJECT> value, <DOUBLE|FLOAT> relative.error, <DOUBLE|FLOAT> confidence)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">value</td>
        <td style="vertical-align: top; word-wrap: break-word">The value for which the count is derived.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>DOUBLE<br>FLOAT<br>LONG<br>STRING<br>BOOL<br>TIME<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">relative.error</td>
        <td style="vertical-align: top; word-wrap: break-word">This is the relative error to be allowed for the count generated, expressed as a value between 0 and 1. Lower the value specified, lower is the rate by which the count can deviate from being perfectly correct. If 0.01 is specified, the count generated must be almost perfectly accurate. If 0.99 is specified, minimal level of accuracy is expected. Note that you cannot specify <code>1</code> or <code>0</code> as the value for this parameter.</td>
        <td style="vertical-align: top">0.01</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">confidence</td>
        <td style="vertical-align: top; word-wrap: break-word">This value determines the rate by which the result can deviate from the actual event count. Higher the value specified, higher is the possibility of the amount of error in the count being no greater than the relative error specified. If 0.99 is specified, it is almost certain that the count is generated with the specified rate of relative error. If 0.01 is specified, there can be minimal certainty as to whether the count is generated with the specified rate of error. Note that you cannot specify <code>1</code> or <code>0</code> as the value for this parameter.</td>
        <td style="vertical-align: top">0.99</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>
<span id="extra-return-attributes" class="md-typeset" style="display: block; font-weight: bold;">Extra Return Attributes</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Possible Types</th>
    </tr>
    <tr>
        <td style="vertical-align: top">count</td>
        <td style="vertical-align: top; word-wrap: break-word">This represents the approximate count per attribute based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">countLowerBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The lowest value in the range within which the most accurate count for the attribute is included. This count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">countUpperBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The highest value in the range within which the most accurate count for the attribute is included. This count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream RequestStream (ip string);
from RequestStream#window.time(1000)#approximate:count(ip)
select count, countLowerBound, countUpperBound
insert into OutputStream;
```
<p style="word-wrap: break-word">This query generates the count(frequency) of requests from different IP addresses in a time window that is calculated with a default relative error of 0.01 and a default confidence of 0.99. Here, only the events that arrive during the last 1000 milliseconds are considered when calculating the counts. The counts generated are 99% guaranteed to deviate from the actual count within the window by only 1%. The output consists of the approximate count of the latest event, lower bound, and upper bound of the approximate answer.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
define stream TransactionStream (userId int, amount double);
from TransactionStream#window.length(1000)#approximate:count(userId, 0.05, 0.9)
select count, countLowerBound, countUpperBound
insert into OutputStream;
```
<p style="word-wrap: break-word">This query generates the count or frequency of transactions for each user ID based on the last 1000 transactions, i.e., events. The counts generated are 90% guaranteed to deviate from the actual event count within the window by only 5%.The output consists of the approximate count of the latest events, lower bound and upper bound of the approximate result.</p>

### distinctCount *<a target="_blank" href="http://siddhi.io/documentation/siddhi-5.x/query-guide-5.x/#stream-processor">(Stream Processor)</a>*

<p style="word-wrap: break-word">This applies the 'HyperLogLog' algorithm to a Siddhi window. The algorithm is set with a relative error and a confidence value on the basis of which the number of distinct events with an accepted level of accuracy is calculated. Note that if this extension is used without a window, it may cause an 'out of memory' error. If you need to perform these calculations without windows, use the <code>approximate:distinctCountEver</code> extension.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
approximate:distinctCount(<INT|DOUBLE|FLOAT|LONG|STRING|BOOL|TIME|OBJECT> value, <DOUBLE|FLOAT> relative.error, <DOUBLE|FLOAT> confidence)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">value</td>
        <td style="vertical-align: top; word-wrap: break-word">The value for which the <code>distinctCount</code> is calculated.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>DOUBLE<br>FLOAT<br>LONG<br>STRING<br>BOOL<br>TIME<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">relative.error</td>
        <td style="vertical-align: top; word-wrap: break-word">This is the relative error to be allowed for the distinct count generated, expressed as a value between 0 and 1. Lower the value specified, lower is the rate by which the distinct count can deviate from being perfectly correct. If 0.01 is specified, the distinct count generated must be almost perfectly accurate. If 0.99 is specified, a minimal level of accuracy is expected. Note that you cannot specify <code>1</code> or <code>0</code> as the value for this parameter.</td>
        <td style="vertical-align: top">0.01</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">confidence</td>
        <td style="vertical-align: top; word-wrap: break-word">The confidence value determines the degree of guarantee with which the relative error given can be treated. Higher the value specified, higher is the possibility of the amount of error in the distinct count being no greater than the relative error specified. If 0.99 is specified, it can be almost considered with certainty that the distinct count is generated with the specified rate of relative error. If 0.01 is specified, there can be minimal certainty as to whether the distinct count is generated with the specified rate of error. The possible values include <code>0.65</code>, <code>0.95</code>, <code>0.99</code>, etc..</td>
        <td style="vertical-align: top">0.95</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>
<span id="extra-return-attributes" class="md-typeset" style="display: block; font-weight: bold;">Extra Return Attributes</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Possible Types</th>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCount</td>
        <td style="vertical-align: top; word-wrap: break-word">This represents the distinct count based on the last event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCountLowerBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The lowest value in the range within which the most accurate distinct count for the attribute is included. This distinct count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCountUpperBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The highest value in the range within which the most accurate distinct count for the attribute is included This distinct count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream RequestStream (ip string);
from RequestStream#window.time(1000)#approximate:distinctCount(ip)
select distinctCount, distinctCountLowerBound, distinctCountUpperBound
insert into OutputStream;

```
<p style="word-wrap: break-word">This query calculates the distinct count of events for each IP address that has sent requests within the last 1000 milliseconds. The distinct count is 95% guaranteed to deviate no more than 1% from the actual distinct count per IP address.The output consists of the approximate distinct count,the lower bound, and the upper bound of the approximate answer.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
define stream SensorStream (sensorId int);
from SensorStream#window.length(1000)
#approximate:distinctCount(sensorId, 0.05, 0.65)
select distinctCount, distinctCountLowerBound, distinctCountUpperBound
insert into OutputStream;

```
<p style="word-wrap: break-word">This query calculates the distinct count of events for each sensor that has sent data to the stream. This value is calculated based on the last 1000 events in a sliding manner. The calculated distinct count is 65% guaranteed to deviate no more than 5% from the actual distinct count. The output consists of the approximate distinct count, and the lower bound and upper bound of the approximate answer.</p>

### distinctCountEver *<a target="_blank" href="http://siddhi.io/documentation/siddhi-5.x/query-guide-5.x/#stream-processor">(Stream Processor)</a>*

<p style="word-wrap: break-word">This extension applies the <code>HyperLogLog</code> algorithm to a Siddhi window in order to calculate the number of distinct events on a streaming data set based on a specific relative error and a confidence value given. Note that this extension returns erroneous values if it is used with a Siddhi window. If you want to perform these calculations with a window, you need to use the <code>approximate:distinctCount</code> extension.</p>

<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>
```
approximate:distinctCountEver(<INT|DOUBLE|FLOAT|LONG|STRING|BOOL|TIME|OBJECT> value, <DOUBLE|FLOAT> relative.error, <DOUBLE|FLOAT> confidence)
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">value</td>
        <td style="vertical-align: top; word-wrap: break-word">The value for which the distinct count is calculated.</td>
        <td style="vertical-align: top"></td>
        <td style="vertical-align: top">INT<br>DOUBLE<br>FLOAT<br>LONG<br>STRING<br>BOOL<br>TIME<br>OBJECT</td>
        <td style="vertical-align: top">No</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">relative.error</td>
        <td style="vertical-align: top; word-wrap: break-word">This is the relative error to be allowed for the distinct count generated, expressed as a value between 0 and 1. Lower the value specified, lower is the rate by which the distinct count can deviate from being perfectly correct. If 0.01 is specified, the distinct count generated must be almost perfectly accurate. If 0.99 is specified, the minimal level of accuracy is expected. Note that you cannot specify <code>1</code> or <code>0</code> as the value for this parameter.</td>
        <td style="vertical-align: top">0.01</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">confidence</td>
        <td style="vertical-align: top; word-wrap: break-word">The confidence value determines the degree of guarantee with which the relative error given can be treated. Higher the value specified, higher is the possibility of the amount of error in the distinct count being no greater than the relative error specified. If 0.99 is specified, it can be almost considered with certainty that the distinct count is generated with the specified rate of relative error. If 0.01 is specified, there can be minimal certainty as to whether the distinct count is generated with the specified rate of error. The possible values include <code>0.65</code>, <code>0.95</code>, <code>0.99</code>, etc..</td>
        <td style="vertical-align: top">0.95</td>
        <td style="vertical-align: top">DOUBLE<br>FLOAT</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>
<span id="extra-return-attributes" class="md-typeset" style="display: block; font-weight: bold;">Extra Return Attributes</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Possible Types</th>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCountEver</td>
        <td style="vertical-align: top; word-wrap: break-word">This represents the distinct count based on the last event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCountEverLowerBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The lowest value in the range within which the most accurate distinct count for the attribute is included. This distinct count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
    <tr>
        <td style="vertical-align: top">distinctCountEverUpperBound</td>
        <td style="vertical-align: top; word-wrap: break-word">The highest value in the range within which the most accurate distinct count for the attribute is included. This distinct count range is based on the latest event.</td>
        <td style="vertical-align: top">LONG</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
define stream RequestStream (ip string);
from RequestStream#approximate:distinctCountEver(ip)
select distinctCountEver, distinctCountEverLowerBound, distinctCountEverUpperBound
insert into OutputStream;

```
<p style="word-wrap: break-word">This query calculates the distinct count of events for each IP address that has sent requests within the last 1000 milliseconds. The distinct count is 95% guaranteed to deviate no more than 1% from the actual distinct count per IP address.The output consists of the approximate distinct count, and the lower bound and upper bound of the approximate answer.</p>

<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
define stream SensorStream (sensorId int);
from SensorStream#approximate:distinctCountEver(sensorId, 0.05, 0.65)
select distinctCountEver, distinctCountEverLowerBound, distinctCountEverUpperBound
insert into OutputStream;

```
<p style="word-wrap: break-word">This query calculates the distinct count of events for each sensor that has sent data to the stream. This value is calculated based on the last 1000 events in a sliding manner. The calculated distinct count is 65% guaranteed to deviate no more than 5% from the actual distinct count. The output consists of the approximate distinct count, the lower bound, and the upper bound of the approximate answer.</p>

